package de.sheepstar.globalWrapper.api;

import de.sheepstar.globalWrapper.api.sql.MySQL;
import net.dv8tion.jda.api.entities.Guild;
import net.dv8tion.jda.api.entities.TextChannel;

import java.sql.ResultSet;
import java.util.ArrayList;
import java.util.Arrays;

/********************************
 * @author Mathias Wagner
 * Created 12.12.20
 ********************************/

public class GlobalManager {

    public boolean isRegistered(Guild guild, TextChannel channel) {
        ResultSet rs = MySQL.getResult("SELECT * FROM guilds WHERE guildID=? AND channelID=?", guild.getId(), channel.getId());
        try {
            while (rs.next()) return true;
        } catch (Exception err) {}
        return false;
    }

    public boolean isDescRegistered(Guild guild, TextChannel channel) {
        ResultSet rs = MySQL.getResult("SELECT * FROM server_autoupdate WHERE guildID=? AND channelID=?", guild.getId(), channel.getId());
        try {
            while (rs.next()) return true;
        } catch (Exception err) {}
        return false;
    }


    public void register(Guild guild, TextChannel channel) {
        if (isRegistered(guild, channel)) return;
        MySQL.update("INSERT INTO guilds (guildID, channelID, languageCode, roomID) VALUES (?, ?, 'EN', '1')", guild.getId(), channel.getId());
    }

    public void unregister(Guild guild, TextChannel channel) {
        if (!isRegistered(guild, channel)) return;
        MySQL.update("DELETE FROM guilds WHERE guildID=? AND channelID=?", guild.getId(), channel.getId());
    }

    public boolean isValidRoom(String str) {
        ResultSet rs = MySQL.getResult("SELECT * FROM rooms WHERE code=?", str);
        try {
            while (rs.next()) return true;
        } catch (Exception err) {}
        return false;
    }

    public boolean hasPremium(Guild guild) {
        ResultSet rs = MySQL.getResult("SELECT * FROM premium WHERE guildID=?", guild.getId());
        try {
            while (rs.next()) return true;
        } catch (Exception err) {}
        return false;
    }

   public boolean isValidLanguage(String str) {
        ResultSet rs = MySQL.getResult("SELECT * FROM languages WHERE code=?", str.toUpperCase());
        try {
            while (rs.next()) return true;
        } catch (Exception err) {}
        return false;
    }

    public void changeLanguageCode(Guild guild, TextChannel channel, String newLanguage) {
        if (!isValidLanguage(newLanguage)) return;
        MySQL.update("UPDATE guilds SET languageCode=? WHERE guildID=? AND channelID=?", newLanguage.toUpperCase(), guild.getId(), channel.getId());
    }

    public void changeRoomID(Guild guild, TextChannel channel, String newRoom) {
        if (!isValidRoom(newRoom)) return;
        MySQL.update("UPDATE guilds SET roomID=? WHERE guildID=? AND channelID=?", newRoom, guild.getId(), channel.getId());
    }

    public void setPremium(Guild guild) {
       if (!hasPremium(guild)) {
           MySQL.update("INSERT INTO premium (guildID) VALUES (?)", guild.getId());
       }
    }

    public String getDesign(Guild guild) {
        ResultSet rs = MySQL.getResult("SELECT * FROM designs WHERE guildID=?", guild.getId());
        try {
            while (rs.next()) {
                return rs.getString("designID");
            }
        } catch (Exception ignore) {}
        return "1";
    }

    public boolean isDesignRegistered(Guild guild) {
        ResultSet rs = MySQL.getResult("SELECT * FROM designs WHERE guildID=?", guild.getId());
        try { while (rs.next()) { return true; } } catch (Exception ignore) {}
        return false;
    }

    public boolean isValidDesign(String designID) {
        return new ArrayList<>(Arrays.asList("1", "2", "3", "4", "5")).contains(designID);
    }

    public void registerDesign(Guild guild, String design) {
        if (!isDesignRegistered(guild) && isValidDesign(design)) {
            MySQL.update("INSERT INTO designs (guildID, designID) VALUES (?, ?)", guild.getId(), design);
        }
    }

    public void updateDesign(Guild guild, String design) {
        if (!isDesignRegistered(guild)) registerDesign(guild, design);
        if (isDesignRegistered(guild) && isValidDesign(design)) {
            MySQL.update("UPDATE designs SET designID=? WHERE guildID=?", design, guild.getId());
        }
    }

    public boolean isBanned(Guild guild) {
        ResultSet rs = MySQL.getResult("SELECT * FROM server_bans WHERE guildID=?", guild.getId());
        try {
            while (rs.next()) return true;
        } catch (Exception err) {}
        return false;
    }

    public void banServer(Guild guild) {
        if (!isBanned(guild)) {
            MySQL.update("INSERT INTO server_bans (guildID) VALUES (?)", guild.getId());
        }
    }

    public void unbanServer(Guild guild) {
        if (isBanned(guild)) {
            MySQL.update("DELETE FROM server_bans WHERE guildID=? ", guild.getId());
        }
    }

    public boolean isGlobalUserBanned(String userID) {
        ResultSet rs = MySQL.getResult("SELECT * FROM user_bans WHERE clientID=?", userID);
        try {
            while (rs.next()) return true;
        } catch (Exception err) {}
        return false;
    }

    public void banUserGlobal(String userID) {
        if (!isGlobalUserBanned(userID)) {
            MySQL.update("INSERT INTO user_bans (clientID) VALUES (?)", userID);
        }
    }

    public void unbanUserGlobal(String userID) {
        if (isGlobalUserBanned(userID)) {
            MySQL.update("DELETE FROM user_bans WHERE clientID=? ", userID);
        }
    }

    public boolean isGuildUserBanned(Guild guild, String userID) {
        ResultSet rs = MySQL.getResult("SELECT * FROM suser_bans WHERE guildID=? AND clientID=?", guild.getId(), userID);
        try {
            while (rs.next()) return true;
        } catch (Exception err) {}
        return false;
    }

    public void banUserGuild(Guild guild, String userID) {
        if (!isGuildUserBanned(guild, userID)) {
            MySQL.update("INSERT INTO suser_bans (guildID, clientID) VALUES (?, ?)", guild.getId(), userID);
        }
    }

    public void unbanUserGuild(Guild guild, String userID) {
        if (isGuildUserBanned(guild, userID)) {
            MySQL.update("DELETE FROM suser_bans WHERE guildID=? AND clientID=? ", guild.getId(), userID);
        }
    }


    public int getCount(String language, String room) {
        int count = 0;
        ResultSet rs = MySQL.getResult("SELECT * FROM guilds WHERE languageCode=? AND roomID=?", language, room);
        try { while (rs.next()) {count++;} } catch (Exception err) {}
        return count;
    }

    public int getCount() {
        int count = 0;
        ResultSet rs = MySQL.getResult("SELECT * FROM guilds");
        try { while (rs.next()) {count++;} } catch (Exception err) {}
        return count;
    }

    public int getCount(Guild guild) {
        int count = 0;
        ResultSet rs = MySQL.getResult("SELECT * FROM guilds WHERE guildID=?", guild.getId());
        try { while (rs.next()) {count++;} } catch (Exception err) {}
        return count;
    }

    public boolean alreadyServerBlacklisted(Guild guild, String word) {
        ResultSet rs = MySQL.getResult("SELECT * FROM sword_blacklist WHERE guildID=? AND word=?", guild.getId(), word);
        try {
            while (rs.next()) return true;
        } catch (Exception err) {}
        return false;
    }

    public boolean alreadyGlobalBlacklisted(String word) {
        ResultSet rs = MySQL.getResult("SELECT * FROM word_blacklist WHERE word=?", word);
        try {
            while (rs.next()) return true;
        } catch (Exception err) {}
        return false;
    }


    public void insertLog(String guildID, String senderID, String message, String language, String roomID) {
        if (message.isEmpty()) message = "empty";
        MySQL.update("INSERT INTO log (guildID, senderID, message, language, roomID) VALUES (?, ?, ?, ?, ?)",
                guildID, senderID, message , language, roomID);
    }

}
