package de.sheepstar.globalWrapper.api;

import de.sheepstar.globalWrapper.api.messagebuilder.MessageBuilder;
import de.sheepstar.globalWrapper.api.messagebuilder.MessageType;
import de.sheepstar.globalWrapper.api.sql.MySQL;
import de.sheepstar.globalWrapper.core.Main;
import net.dv8tion.jda.api.entities.Guild;
import net.dv8tion.jda.api.entities.TextChannel;

import java.math.BigDecimal;
import java.sql.ResultSet;
import java.util.ArrayList;

/********************************
 * @author Mathias Wagner
 * Created 18.12.20
 ********************************/

public class LevelManager {

    private Guild guild;

    public LevelManager(Guild guild) {
        this.guild = guild;
    }


    public void updateXP(String newXP) {
        if (!isRegistered()) register();
        MySQL.update("UPDATE levels SET xp=? WHERE guildID=?", newXP, guild.getId());
    }

    public void increaseXP(String addXP) {
        if (!isRegistered()) register();
        MySQL.update("UPDATE levels SET xp=xp+? WHERE guildID=?", addXP, guild.getId());
    }

    public void updateLevel(String newLevel) {
        if (!isRegistered()) register();
        MySQL.update("UPDATE levels SET level=? WHERE guildID=?", newLevel, guild.getId());
    }

    public void removeLevel(String removeLevel) {
        if (!isRegistered()) register();
        MySQL.update("UPDATE levels SET level=level-? WHERE guildID=?", removeLevel, guild.getId());
    }

    public void increaseLevel(String addLevel) {
        if (!isRegistered()) register();
        MySQL.update("UPDATE levels SET level=level+? WHERE guildID=?", addLevel, guild.getId());
    }

    public void increase(TextChannel channel, String addXP) {
        increaseXP(addXP);
        int currentXP = Integer.parseInt(getXP());
        int neededXP = getNeededXP();
        if (currentXP >= neededXP) {
            levelUp(channel);
            if (currentXP != 0) updateXP(String.valueOf(currentXP - neededXP));
        }
    }

    public int getNeededXP() {
        return Integer.parseInt(getLevel())*50+5;
    }

    private boolean isRegistered() {
        ResultSet rs = MySQL.getResult("SELECT * FROM levels WHERE guildID=?", guild.getId());
        try { while (rs.next()) { return true; } } catch (Exception ignored) {}
        return false;
    }

    private void register() {
        if (isRegistered()) return;
        MySQL.update("INSERT INTO levels (guildID, xp, level) VALUES (?, '0', '0')", guild.getId());
    }

    public void levelUp(TextChannel channel) {
        if (!isRegistered()) register();
        MySQL.update("UPDATE levels SET level=level+1 WHERE guildID=?", guild.getId());
        MySQL.update("UPDATE levels SET xp=0 WHERE guildID=?", guild.getId());
        new MessageBuilder(MessageType.GREEN_MESSAGE)
                .setTitle("Level up!")
                .setDescription("You reached level " + getLevel())
                .send(channel);
    }

    public String getXP() {
        if (!isRegistered()) register();
        ResultSet rs = MySQL.getResult("SELECT * FROM levels WHERE guildID=?", guild.getId());
        try { while (rs.next()) { return rs.getString("xp"); } } catch (Exception ignored) {}
        return "0";
    }

    public String getLevel() {
        if (!isRegistered()) register();
        ResultSet rs = MySQL.getResult("SELECT * FROM levels WHERE guildID=?", guild.getId());
        try { while (rs.next()) { return rs.getString("level"); } } catch (Exception ignored) {}
        return "0";
    }

    public String getLevel(Guild guild) {
        if (!isRegistered()) register();
        ResultSet rs = MySQL.getResult("SELECT * FROM levels WHERE guildID=?", guild.getId());
        try { while (rs.next()) { return rs.getString("level"); } } catch (Exception ignored) {}
        return "0";
    }

    public ArrayList<Guild> getTopList(int limit) {
        ResultSet rs = MySQL.getResult("SELECT * FROM levels ORDER BY `level` DESC LIMIT " + limit);
        ArrayList<Guild> topList = new ArrayList<>();
        try { while (rs.next()) { try { topList.add(Main.getInstance().getGuildById(rs.getString("guildID"))); } catch (Exception ignored) { } } } catch (Exception ignored) { }
        return topList;
    }

    public Guild getGuild() {
        return guild;
    }
}
