package de.sheepstar.globalWrapper.api;

import de.sheepstar.globalWrapper.api.messagebuilder.MessageBuilder;
import de.sheepstar.globalWrapper.api.messagebuilder.MessageType;
import de.sheepstar.globalWrapper.core.Main;
import net.dv8tion.jda.api.JDA;
import net.dv8tion.jda.api.Permission;
import net.dv8tion.jda.api.entities.Guild;
import net.dv8tion.jda.api.entities.Message;
import net.dv8tion.jda.api.events.message.guild.GuildMessageReceivedEvent;
import net.dv8tion.jda.api.hooks.ListenerAdapter;

import java.util.List;

/********************************
 * @author Mathias Wagner
 * Created 11.12.20
 ********************************/

public abstract class SheepstarGuildCommand extends ListenerAdapter {

    public String replaceUserMention(String content) {
        return content
                .replace("<@!", "")
                .replace("<@", "")
                .replace("<!", "")
                .replace(">", "");
    }

    public String getPrefix() {
        if (SheepManager.BETA_INSTANCE) {
            return "st!";
        } else {
            return "s!";
        }
    }

    public StrangerManager getStrangerManager() {
        return Main.getStrangerManager();
    }

    public GlobalManager getGlobalManager() {
        return Main.getGlobalManager();
    }

    public MessageBuilder sendErrorMessage() {
        return new MessageBuilder(MessageType.RED_MESSAGE);
    }

    public MessageBuilder sendMessage() {
        return new MessageBuilder(MessageType.GREEN_MESSAGE);
    }

    public JDA getJDA() { return Main.getInstance(); }

    public abstract List<String> getAlias();

    public abstract SheepPermission getPermission();

    public abstract void executeCommand(String[] args, GuildMessageReceivedEvent e);

    @Override
    public void onGuildMessageReceived(GuildMessageReceivedEvent e) {
        if (e.getAuthor() == null || e.getChannel() == null) return;
        if (e.getAuthor().isBot()) return;
        if (!isValidCommand(e.getGuild(), e.getMessage())) return;

        if (!getPermission().equals(null)) {

            if (getPermission() == SheepPermission.ADMINISTRATOR) {
                if (!e.getMember().hasPermission(Permission.ADMINISTRATOR)) {
                    sendErrorMessage()
                            .setDescription("You dont have the permission to do this.")
                            .send(e.getChannel());
                    return;
                }
            } else if (getPermission() == SheepPermission.BOT_OWNER) {
                if (!Main.getSheepManager().botOwners.contains(e.getAuthor().getId())) {
                    sendErrorMessage()
                            .setDescription("Only Bot Owners have Permission for this Command")
                            .send(e.getChannel());
                    return;
                }
            } else if (getPermission() == SheepPermission.SUPPORTER) {
                if (!(e.getMember().getRoles().contains(Main.getSheepManager().supporterRole) || Main.getSheepManager().botOwners.contains(e.getAuthor().getId()))) {
                    sendErrorMessage()
                            .setDescription("You dont have the permission to do this.")
                            .send(e.getChannel());
                    return;
                }
            }
        }

        String[] args = commandArgs(e.getGuild(), e.getMessage());
        executeCommand(args, e);
    }

    private boolean isValidCommand(Guild guild, Message msg) {
        String prefix = getPrefix();
        if (!msg.getContentRaw().startsWith(prefix)) return false;
        String cmdName = msg.getContentRaw().substring(prefix.length()).toLowerCase();
        if (cmdName.contains(" ")) {
            cmdName = cmdName.substring(0, cmdName.indexOf(" "));
        }
        if (cmdName.contains("\n")) {
            cmdName = cmdName.substring(0, cmdName.indexOf("\n"));
        }
        return getAlias().contains(cmdName);
    }

    private String[] commandArgs(Guild guild, Message msg) {
        String prefix = getPrefix();
        String noPrefix = msg.getContentRaw().substring(prefix.length());
        if (!noPrefix.contains(" ")) {
            return new String[]{};
        }
        return noPrefix.substring(noPrefix.indexOf(" ") + 1).split("\\s+");
    }

}