package de.sheepstar.globalWrapper.api;

import de.sheepstar.globalWrapper.api.messagebuilder.MessageBuilder;
import de.sheepstar.globalWrapper.api.messagebuilder.MessageType;
import de.sheepstar.globalWrapper.api.sql.MySQL;
import de.sheepstar.globalWrapper.core.Main;
import net.dv8tion.jda.api.entities.Guild;
import net.dv8tion.jda.api.entities.TextChannel;

import java.sql.ResultSet;
import java.util.Objects;
import java.util.Random;

/********************************
 * @author Mathias Wagner
 * Created 14.12.20
 ********************************/

public class StrangerManager {

    public boolean isRegistered(Guild guild, TextChannel channel) {
        ResultSet rs = MySQL.getResult("SELECT * FROM strangers WHERE guildID=? AND channelID=?", guild.getId(), channel.getId());
        try { while (rs.next()) { return true; } } catch (Exception err) {}
        return false;
    }

    public void register(Guild guild, TextChannel channel, String languageCode) {
        if (isRegistered(guild, channel)) return;
        MySQL.update("INSERT INTO strangers (guildID, channelID, language) VALUES (?, ?, ?)", guild.getId(), channel.getId(), languageCode);
    }

    public void unregister(Guild guild, TextChannel channel) {
        if (!isRegistered(guild, channel)) return;
        MySQL.update("DELETE FROM strangers WHERE guildID=? AND channelID=?", guild.getId(), channel.getId());
    }

    public int getCount() {
        int count = 0;
        ResultSet rs = MySQL.getResult("SELECT * FROM strangers");
        try { while (rs.next()) {count++;} } catch (Exception err) {}
        return count;
    }

    public int getCount(Guild guild) {
        int count = 0;
        ResultSet rs = MySQL.getResult("SELECT * FROM strangers WHERE guildID=?", guild.getId());
        try { while (rs.next()) {count++;} } catch (Exception err) {}
        return count;
    }

    public void disconnect(TextChannel textChannel, String connectID) {
        ResultSet rs = MySQL.getResult("SELECT * FROM strangers WHERE connectID=?", connectID);
        try {
            while (rs.next()) {
                new MessageBuilder(MessageType.GREEN_MESSAGE)
                        .setTitle("**Good bye!**")
                        .setDescription("Connection closed.")
                        .send(Objects.requireNonNull(Main.getInstance().getTextChannelById(rs.getString("channelID"))));
                if (!textChannel.getId().equals(rs.getString("channelID"))) {
                    // auto reconnect
                    new MessageBuilder(MessageType.GREEN_MESSAGE)
                            .setTitle("Reconnecting...")
                            .setDescription("Waiting until an other server is connecting to yours")
                            .send(Objects.requireNonNull(Main.getInstance().getTextChannelById(rs.getString("channelID"))));
                    connect(
                            Main.getInstance().getGuildById(rs.getString("guildID")),
                            Main.getInstance().getTextChannelById(rs.getString("channelID")),
                            rs.getString("language")
                    );
                }
            }
        } catch (Exception err) {}
        MySQL.update("UPDATE strangers SET connectID='' WHERE connectID=?", connectID);
    }

    public String getConnectID(Guild guild, TextChannel channel) {
        ResultSet rs = MySQL.getResult("SELECT * FROM strangers WHERE guildID=? AND channelID=?", guild.getId(), channel.getId());
        try {
            while (rs.next()) { return rs.getString("connectID"); }
        } catch (Exception err) {}
        return "null";
    }

    public String getLanguage(Guild guild, TextChannel channel) {
        ResultSet rs = MySQL.getResult("SELECT * FROM strangers WHERE guildID=? AND channelID=?", guild.getId(), channel.getId());
        try {
            while (rs.next()) {
                return rs.getString("language");
            }
        } catch (Exception err) {}
        return "EN";
    }

    public void connect(Guild guild, TextChannel channel, String languageCode) {
        ResultSet rs = MySQL.getResult("SELECT * FROM strangers WHERE connectID='' AND language=? ORDER BY RAND() LIMIT 2", languageCode);
        boolean connected = false;
        String connectID = String.valueOf(new Random().nextInt(999999999));
        try {
            while (rs.next()) {
                String currentGuildID = rs.getString("guildID");
                String channelID = rs.getString("channelID");
                int id=rs.getInt("id");
                if (!connected) {
                    if (!(currentGuildID.equals(guild.getId()) && channelID.equals(channel.getId()))) {
                        MySQL.update("UPDATE strangers SET connectID=? WHERE id=?", connectID, String.valueOf(id));
                        MySQL.update("UPDATE strangers SET connectID=? WHERE guildID=? AND channelID=?", connectID, guild.getId(), channel.getId());
                        new MessageBuilder(MessageType.GREEN_MESSAGE)
                                .setTitle("Connected!")
                                .setDescription("You are connected with `"+Main.getInstance().getGuildById(currentGuildID).getName()+"`")
                                .send(channel);
                        new MessageBuilder(MessageType.GREEN_MESSAGE)
                                .setTitle("Connected!")
                                .setDescription("You are connected with `"+guild.getName()+"`")
                                .send(Objects.requireNonNull(Main.getInstance().getTextChannelById(channelID)));
                        connected = true;
                    }
                }

            }
        } catch (Exception err) {}
        if (!connected) {
            new MessageBuilder(MessageType.RED_MESSAGE)
                    .setTitle("Currently all servers are connected")
                    .setDescription("You are now in the queue, please wait until another server connects with yours")
                    .send(channel);
        }
    }

}
