package de.sheepstar.globalWrapper.api.messagebuilder;

import de.sheepstar.globalWrapper.api.SheepManager;
import net.dv8tion.jda.api.EmbedBuilder;
import net.dv8tion.jda.api.entities.Guild;
import net.dv8tion.jda.api.entities.Message;
import net.dv8tion.jda.api.entities.MessageEmbed;
import net.dv8tion.jda.api.entities.TextChannel;

import java.awt.*;
import java.time.OffsetDateTime;
import java.time.ZoneId;

/********************************
 * @author Mathias Wagner
 * Created 12.12.20
 ********************************/

public class MessageBuilder {

    EmbedBuilder eb;

    public MessageBuilder(MessageType type) {
        eb = new EmbedBuilder();
        setThumbnail("https://global.sheepstar.xyz/Sstar_Global_Rund.png");
        if (type == MessageType.GREEN_MESSAGE) {
            eb.setColor(new Color(0x2ecc71));
        } else if (type == MessageType.RED_MESSAGE) {
            eb.setColor(new Color(0xde1f1f));
            eb.setTitle("Oh no!");
        }
    }

    public MessageBuilder addField(String name, String value, boolean inline) {
        eb.addField(name, value, inline);
        return this;
    }

    public MessageBuilder setTitle(String title) {
        eb.setTitle(title);
        return this;
    }

    public MessageBuilder setDescription(String description) {
        description = (SheepManager.BETA_INSTANCE) ? description.replace("s!", "st!") : description;
        eb.setDescription(description);
        return this;
    }

    public MessageBuilder setAuthor(String author) {
        eb.setAuthor(author);
        return this;
    }

    public MessageBuilder setThumbnail(String url) {
        eb.setThumbnail(url);
        return this;
    }

    public MessageBuilder setFooter(String footer) {
        footer = (SheepManager.BETA_INSTANCE) ? footer.replace("s!", "st!") : footer;
        eb.setFooter(footer);
        return this;
    }

    public MessageBuilder enableTimeStamp() {
        eb.setTimestamp(OffsetDateTime.now().atZoneSameInstant(ZoneId.of("Europe/Berlin")));
        return this;
    }

    public MessageEmbed get() {
        return eb.build();
    }

    public MessageBuilder send(TextChannel channel) {
        channel.sendMessage(eb.build()).queue();
        return this;
    }

}
