package de.sheepstar.globalWrapper.commands;

import de.sheepstar.globalWrapper.api.LevelManager;
import de.sheepstar.globalWrapper.api.SheepPermission;
import de.sheepstar.globalWrapper.api.SheepstarGuildCommand;
import de.sheepstar.globalWrapper.api.ShopManager;
import net.dv8tion.jda.api.Permission;
import net.dv8tion.jda.api.events.message.guild.GuildMessageReceivedEvent;

import java.util.Arrays;
import java.util.List;

/********************************
 * @author Mathias Wagner
 * Created 11.12.20
 ********************************/

public class AddGlobalCommand extends SheepstarGuildCommand {
    @Override
    public List<String> getAlias() {
        return Arrays.asList("addglobal", "newglobal", "ag");
    }

    @Override
    public SheepPermission getPermission() {
        return SheepPermission.ADMINISTRATOR;
    }

    @Override
    public void executeCommand(String[] args, GuildMessageReceivedEvent e) {
        if (getStrangerManager().isRegistered(e.getGuild(), e.getChannel())) {
            sendErrorMessage()
                    .setTitle("Wait a minute...")
                    .setDescription("You already added this as Connection on your server. Disconnect with `s!disconnect`")
                    .send(e.getChannel());
            return;
        }
        if (getGlobalManager().isRegistered(e.getGuild(), e.getChannel())) {
            sendErrorMessage()
                    .setTitle("Wait a minute... ")
                    .setDescription("You already have this GlobalChat on your server.\n" +
                                    "Please note that each server can only have one GlobalChat per room.")
                    .send(e.getChannel());

        } else {
            ShopManager sm = new ShopManager(new LevelManager(e.getGuild()));
            boolean isAllowed = false;
            if (getGlobalManager().hasPremium(e.getGuild())) isAllowed = true;
            if (getGlobalManager().getCount(e.getGuild()) < sm.getItemCount("channel")) isAllowed = true;
            if (getGlobalManager().getCount(e.getGuild()) == 0) isAllowed = true;

            if (!isAllowed) {
                sendErrorMessage()
                        .setDescription("You need Premium or the `channel-item` to add more GlobalChats. You can buy these things under s!shop")
                        .send(e.getChannel());
                return;
            }
            getGlobalManager().register(e.getGuild(), e.getChannel());

            sendMessage()
                    .setTitle("**__Welcome to the Global Chat of Sheepstar__**")
                    .setDescription("**:warning: Warning**: \n" +
                            "Everything written here will be sent to all servers connected to <@772513556116930580>. " +
                            "For questions and help, contact our team on the [Support Server](https://discord.gg/jbne9JTJtU) \n" +
                            "Furthermore, you and your server team have the duty to make sure that the rules in the Global Chat are followed! \n" +
                            "If this is not the case, your server will be immediately excluded from Global Chat.")
                    .send(e.getChannel());
        }
    }
}
