package de.sheepstar.globalWrapper.commands;

import de.sheepstar.globalWrapper.api.SheepManager;
import de.sheepstar.globalWrapper.api.SheepPermission;
import de.sheepstar.globalWrapper.api.SheepstarGuildCommand;
import de.sheepstar.globalWrapper.api.sql.MySQL;
import de.sheepstar.globalWrapper.core.Main;
import net.dv8tion.jda.api.events.message.guild.GuildMessageReceivedEvent;

import java.sql.ResultSet;
import java.util.Arrays;
import java.util.List;

/********************************
 * @author Mathias Wagner
 * Created 11.12.20
 ********************************/

public class AddStrangerCommand extends SheepstarGuildCommand {
    @Override
    public List<String> getAlias() {
        return Arrays.asList("connect", "connection", "newconnection");
    }

    @Override
    public SheepPermission getPermission() {
        return SheepPermission.ADMINISTRATOR;
    }

    @Override
    public void executeCommand(String[] args, GuildMessageReceivedEvent e) {
        String language = "EN";
        if (args.length == 1) {
            if (getGlobalManager().isValidLanguage(args[0])) {
                language = args[0].toUpperCase();
            } else {
                sendErrorMessage()
                        .setDescription("This is not a valid language. See a list under `s!langs`")
                        .send(e.getChannel());
            }
        }
        if (getGlobalManager().isRegistered(e.getGuild(), e.getChannel())) {
            sendErrorMessage()
                    .setTitle("Uhm... Nope")
                    .setDescription("You already added this channel as GlobalChat on your server. Remove with `s!removeGlobal`")
                    .send(e.getChannel());
            return;
        }

        if (getStrangerManager().isRegistered(e.getGuild(), e.getChannel())) {
          sendErrorMessage()
                  .setTitle("Wait a minute...")
                  .setDescription("You already added this as Connection on your server. Disconnect with `s!disconnect`")
                  .send(e.getChannel());
            return;
        }

        if (getStrangerManager().getCount(e.getGuild()) > 0 && !Main.getSheepManager().botOwners.contains(e.getAuthor().getId())) {
            sendErrorMessage()
                    .setTitle("Connection maximum reached")
                    .setDescription("You are not allowed to create more of these channels.")
                    .send(e.getChannel());
            return;
        }

        getStrangerManager().register(e.getGuild(), e.getChannel(), language);

        sendMessage()
                .setTitle("**__You are now in the queue, please wait until another server connects with your server.__**")
                .setDescription("**:warning: Warning**: \n" +
                        "Everything written here will be sent to another server, connected to this function by <@772513556116930580>. " +
                        "For questions and help, contact our team on the [Support Server](https://discord.gg/jbne9JTJtU) \n" +
                        "Also you and your server team have the duty to make sure that the rules in the 'Stranger Area' are followed! \n" +
                        "If this is not the case, your server will be immediately excluded from this Bot. " +
                        "Also if you see other servers not following the rules you have to report them to the team.")
                .send(e.getChannel());


        getStrangerManager().connect(e.getGuild(), e.getChannel(), language);
    }
}
