package de.sheepstar.globalWrapper.commands;

import de.sheepstar.globalWrapper.api.*;
import net.dv8tion.jda.api.events.message.guild.GuildMessageReceivedEvent;

import java.util.Arrays;
import java.util.List;

/********************************
 * @author Mathias Wagner
 * Created 19.12.20
 ********************************/

public class BuyCommand extends SheepstarGuildCommand {
    @Override
    public List<String> getAlias() {
        return Arrays.asList("buy", "buyitem", "buyit");
    }

    @Override
    public SheepPermission getPermission() {
        return SheepPermission.ADMINISTRATOR;
    }

    @Override
    public void executeCommand(String[] args, GuildMessageReceivedEvent e) {
        LevelManager lm = new LevelManager(e.getGuild());
        ShopManager sm = new ShopManager(lm);
        if (args.length != 1) {
            sendErrorMessage()
                    .setDescription("Please provide a product. Look at the list under s!shop")
                    .send(e.getChannel());
            return;
        }
        if (!ShopItems.shopItems.containsKey(args[0])) {
            sendErrorMessage()
                    .setDescription("This product does not exist. Look at the list under s!shop")
                    .send(e.getChannel());
            return;
        }

        ShopItem item = ShopItems.shopItems.get(args[0]);

        int price = getGlobalManager().hasPremium(e.getGuild()) ? Integer.parseInt(item.getPremiumPrice()) : Integer.parseInt(item.getPrice());

        if (!item.isMultiple_buyable() && sm.hasItem(args[0].toLowerCase())) {
            sendErrorMessage()
                    .setDescription("You already bought this Item.")
                    .send(e.getChannel());
            return;
        }

        if (Integer.parseInt(lm.getLevel()) >= price) {
            sm.buyItem(String.valueOf(price), args[0].toLowerCase(), !item.isMultiple_buyable());
            sendMessage()
                    .setTitle("Item bought")
                    .setDescription("The item " + args[0].toLowerCase() + " was bought successfully.")
                    .send(e.getChannel());
        } else {
            sendErrorMessage()
                    .setDescription("You dont habe enough levels to buy this item")
                    .send(e.getChannel());
        }


    }
}
