package de.sheepstar.globalWrapper.commands;

import de.sheepstar.globalWrapper.api.SheepPermission;
import de.sheepstar.globalWrapper.api.SheepstarGuildCommand;
import de.sheepstar.globalWrapper.api.sql.MySQL;
import de.sheepstar.globalWrapper.core.Main;
import net.dv8tion.jda.api.Permission;
import net.dv8tion.jda.api.events.message.guild.GuildMessageReceivedEvent;

import java.sql.ResultSet;
import java.util.Arrays;
import java.util.List;

/********************************
 * @author Mathias Wagner
 * Created 13.12.20
 ********************************/

public class ViewLogCommand extends SheepstarGuildCommand {
    @Override
    public List<String> getAlias() {
        return Arrays.asList("log", "viewlog", "vl", "l");
    }

    @Override
    public SheepPermission getPermission() {
        return SheepPermission.SUPPORTER;
    }

    @Override
    public void executeCommand(String[] args, GuildMessageReceivedEvent e) {
        if (!(args.length == 3 ||args.length == 2)) {
            sendErrorMessage()
                    .setDescription("Use s!log <user/guild/search> <guildID/userID/word> <limit>")
                    .send(e.getChannel());
            return;
        }
        if (args[0].toLowerCase().equals("user") && args.length == 3) {
            args[1] = replaceUserMention(args[1]);
            String content = "";
            int count = 0;
            for (String current : Main.getLogManager().getUserMessages(args[1], Integer.parseInt(args[2]))) {
                if (count != 0) content += "\n";
                count++;
                content += "`"+current+"`";
            }
            String finalContent = content;
            Main.getInstance().retrieveUserById(args[1]).queue(user -> {
                try {
                    sendMessage()
                            .setTitle("Message user-log ["+user.getAsTag()+"]")
                            .setDescription(finalContent)
                            .send(e.getChannel());
                } catch (Exception err) {
                    sendErrorMessage()
                            .setDescription("User does not exist!")
                            .send(e.getChannel());
                }

            });
        } else if (args[0].toLowerCase().equals("guild") && args.length == 3) {
            String content = "";
            int count = 0;
            for (String current : Main.getLogManager().getServerMessages(args[1], Integer.parseInt(args[2]))) {
                if (count != 0) content += "\n";
                count++;
                content += "`"+current+"`";
            }
            sendMessage()
                    .setTitle("Message server-log")
                    .setDescription(content)
                    .send(e.getChannel());
        } else if (args[0].toLowerCase().equals("search")) {
            String content = "";
            String limitLine = "";
            if (args.length != 3) {
                limitLine = "";
            } else {
                limitLine = " LIMIT " + args[2];
            }
            ResultSet rs = MySQL.getResult("SELECT * from log WHERE message REGEXP ? ORDER BY id DESC" + limitLine, args[1].replace("_", " "));

            int count = 0;
            try {
                while (rs.next()) {
                    if (count != 0) content += "\n\n";
                    count++;
                    content += "`"+rs.getString("message")+"`\n";
                    content += "*Gesendet von " + rs.getString("senderID") + " vom Server " + rs.getString("guildID") + "*";
                }
            } catch (Exception err) {}
            if (content.length() > 2000) {
                content = content.substring(0, Math.min(content.length(), 2000)) + "`...`";
            }
            content = "`" +count + " Results`\n\n" + content + "\n\n`" +count + " Results`";
            sendMessage()
                    .setTitle("Message search")
                    .setDescription(content)
                    .send(e.getChannel());
        }
    }
}
