package de.sheepstar.globalWrapper.core;


import de.sheepstar.globalWrapper.api.*;
import de.sheepstar.globalWrapper.api.sql.MySQL;
import de.sheepstar.globalWrapper.api.sql.TableGenerator;
import de.sheepstar.globalWrapper.commands.*;
import de.sheepstar.globalWrapper.instance.AutoUpdater;
import de.sheepstar.globalWrapper.instance.DescUpdater;
import de.sheepstar.globalWrapper.listeners.RoomReactionListener;
import de.sheepstar.globalWrapper.listeners.GlobalListener;
import de.sheepstar.globalWrapper.listeners.StrangerListener;
import net.dv8tion.jda.api.JDA;
import net.dv8tion.jda.api.JDABuilder;
import net.dv8tion.jda.api.requests.GatewayIntent;

import java.util.Random;

/********************************
 * @author Mathias Wagner
 * Created 11.12.20
 ********************************/

public class Main {

    private static JDA jda;
    private static SheepManager sheepManager;
    private static GlobalManager globalManager;
    private static LogManager logManager;
    private static StrangerManager strangerManager;
    public static void main(String[] args) throws Exception {
        sheepManager = new SheepManager().register();
        if (SheepManager.BETA_INSTANCE) {
            SheepManager.sendConsoleMessage("Warnung! Beta-Instanz wird verwendet; Der Bot startet in 3 Sekunden. :)");
            Thread.sleep(3000);
        }
        SheepManager.sendConsoleMessage("Bot startet nun...");
        MySQL.connect();
        TableGenerator.run();
        globalManager = new GlobalManager();
        strangerManager = new StrangerManager();
        logManager = new LogManager();
        jda = JDABuilder.createDefault(sheepManager.getToken())
                .enableIntents(GatewayIntent.GUILD_MEMBERS)
                .build();
        ShopItems.register();

        jda.addEventListener(new ExampleCommand());
        jda.addEventListener(new HelpCommand());
        jda.addEventListener(new RulesCommand());

        jda.addEventListener(new AddGlobalCommand());
        jda.addEventListener(new RemoveGlobalCommand());
        jda.addEventListener(new ClearGlobalCommand());

        jda.addEventListener(new GlobalListener());

        jda.addEventListener(new ShowLanguagesCommand());
        jda.addEventListener(new ChangeLanguageCommand());

        jda.addEventListener(new ShowRoomsCommand());
        jda.addEventListener(new ChangeRoomCommand());

        jda.addEventListener(new AddOffenseCommand());
        jda.addEventListener(new RemoveOffenseCommand());

        jda.addEventListener(new SetPremiumCommand());

        jda.addEventListener(new BanServerCommand());
        jda.addEventListener(new UnbanServerCommand());
        jda.addEventListener(new BanUserCommand());
        jda.addEventListener(new UnbanUserCommand());
        jda.addEventListener(new ViewLogCommand());
        jda.addEventListener(new PingCommand());
        jda.addEventListener(new RoomReactionListener());
        jda.addEventListener(new AutoUpdateCommand());
        jda.addEventListener(new AddStrangerCommand());
        jda.addEventListener(new RemoveStrangerCommand());
        jda.addEventListener(new SkipConnectionCommand());
        jda.addEventListener(new StrangerListener());
        jda.addEventListener(new ShowDesignsCommand());
        jda.addEventListener(new DesignCommand());
        jda.addEventListener(new LevelCommand());
        jda.addEventListener(new LeaderboardCommand());
        jda.addEventListener(new ShopCommand());
        jda.addEventListener(new BuyCommand());
        jda.awaitReady();
        sheepManager.supporterRole = Main.getInstance().getGuildById("772528034912927756").getRoleById("772528035387539476");
        Thread autoUpdater = new Thread(new AutoUpdater());
        autoUpdater.start();
        Thread descUpdater = new Thread(new DescUpdater());
        descUpdater.start();
        SheepManager.sendConsoleMessage("Bot gestartet");
    }

    public static JDA getInstance() {
        return jda;
    }

    public static SheepManager getSheepManager() {
        return sheepManager;
    }

    public static GlobalManager getGlobalManager() {
        return globalManager;
    }

    public static LogManager getLogManager() {
        return logManager;
    }

    public static StrangerManager getStrangerManager() {
        return strangerManager;
    }
}
