package de.sheepstar.globalWrapper.listeners;

import de.sheepstar.globalWrapper.api.*;
import de.sheepstar.globalWrapper.api.messagebuilder.MessageBuilder;
import de.sheepstar.globalWrapper.api.messagebuilder.MessageType;
import de.sheepstar.globalWrapper.api.sql.MySQL;
import de.sheepstar.globalWrapper.core.Main;
import net.dv8tion.jda.api.EmbedBuilder;
import net.dv8tion.jda.api.entities.Guild;
import net.dv8tion.jda.api.entities.TextChannel;
import net.dv8tion.jda.api.events.message.guild.GuildMessageReceivedEvent;
import net.dv8tion.jda.api.hooks.ListenerAdapter;

import java.awt.*;
import java.sql.ResultSet;
import java.time.OffsetDateTime;
import java.time.ZoneId;
import java.util.Random;

/********************************
 * @author Mathias Wagner
 * Created 12.12.20
 ********************************/

public class GlobalListener extends ListenerAdapter {

    @Override
    public void onGuildMessageReceived(GuildMessageReceivedEvent e) {
        GlobalManager gm = Main.getGlobalManager();
        LevelManager lm = new LevelManager(e.getGuild());
        ShopManager sm = new ShopManager(lm);

        // Check if GlobalChat is registered
        if (!gm.isRegistered(e.getGuild(), e.getChannel())) return;

        // Run validation checks
        if (!new ValidationCheck().validate(e)) return; 

        // Preset LanguageCode & RoomID
        ResultSet rs1 = MySQL.getResult("SELECT * FROM guilds WHERE guildID=? AND channelID=?", e.getGuild().getId(), e.getChannel().getId());
        String langcode = "";
        String roomID = "";
        try { while (rs1.next()) { langcode = rs1.getString("languageCode");roomID = rs1.getString("roomID"); } } catch (Exception err) {}

        // NSFW Check
        boolean mustNSFW = roomID.contains("n");
        if (!e.getChannel().isNSFW() && mustNSFW) {
            new MessageBuilder(MessageType.RED_MESSAGE)
                    .setDescription("You joined an NSFW Room. Please set this Channel to NSFW or change the Room with `s!rooms`")
                    .send(e.getChannel());
            return;
        }

        // Send Message To all Servers
        sendGlobal(e, langcode, roomID);

        // Increase Level

        int memberCount = e.getGuild().getMemberCount();
        int extraMultiplier = 1;
        if (memberCount <= 2500) extraMultiplier = 2;
        if (memberCount <= 1000) extraMultiplier = 3;
        if (memberCount <= 10) extraMultiplier = 4;

        int possibility = new Random().nextInt(99);
        if (possibility == 50) {
            lm.increase(e.getChannel(), "15");
        } else {
            int multiplier = sm.getItemCount("booster");
            multiplier += 1;
            lm.increase(e.getChannel(), String.valueOf(new Random().nextInt(5*multiplier*extraMultiplier)));
        }

        // Insert it to Log
        gm.insertLog(e.getGuild().getId(), e.getAuthor().getId(), e.getMessage().getContentRaw(), langcode, roomID);
    }


    public void sendGlobal(GuildMessageReceivedEvent e, String langcode, String roomID) {
        ResultSet rs = MySQL.getResult("SELECT * FROM guilds WHERE languageCode=? AND roomID=?", langcode, roomID);
        GlobalManager gm = Main.getGlobalManager();
        try {
            while (rs.next()) {
                try {
                    // Preset Server Variables
                    Guild currentGuild = Main.getInstance().getGuildById(rs.getString("guildID"));
                    TextChannel currentChannel = currentGuild.getTextChannelById(rs.getString("channelID"));
                    boolean letItWork = true;
                    if (!currentChannel.isNSFW() && roomID.startsWith("n")) letItWork = false;
                    if (letItWork) {
                        if (gm.getDesign(currentGuild).equals("4")) {
                            currentChannel.sendMessage(
                                    "[" + e.getAuthor().getAsTag() + "] " + e.getMessage().getContentRaw()
                            ).queue();
                        } else {
                            EmbedBuilder eb = new EmbedBuilder();
                            String iconUrl = e.getGuild().getIconUrl();
                            if (iconUrl == null) iconUrl = "https://i.giphy.com/media/38pPhtupwYMerEQeBT/source.gif";
                            eb.setThumbnail(iconUrl);
                            eb.setTimestamp(OffsetDateTime.now().atZoneSameInstant(ZoneId.of("Europe/Berlin")));
                            if (roomID.startsWith("n")) {
                                eb.addField(" ", "`NSFW` :underage:", false);
                            } else {
                                if (!(gm.getDesign(currentGuild).equals("2") || gm.getDesign(currentGuild).equals("3"))) {
                                    eb.addField(" ", " ", false);
                                }
                            }
                            if (!gm.getDesign(currentGuild).equals("3")) {
                                String links = "[Support Server](https://discord.gg/jbne9JTJtU) ║ [Bot Invite](https://discord.com/api/oauth2/authorize?client_id=772513556116930580&permissions=321536&scope=bot)";
                                eb.addField("Links & Help", links, false);
                            }
                            if (!gm.getDesign(currentGuild).equals("5")) {
                                eb.setFooter("Sent by " + e.getGuild().getName(), iconUrl);
                            } else {
                                eb.setFooter("Sent by " + e.getGuild().getId() + " from " + e.getAuthor().getId(), iconUrl);
                            }
                            if (e.getMessage().getAttachments().size() > 0)
                                eb.setImage(e.getMessage().getAttachments().get(0).getUrl());

                            eb.setDescription(e.getMessage().getContentRaw());
                            eb.setAuthor(e.getAuthor().getAsTag(), null, e.getAuthor().getAvatarUrl());
                            eb.setColor(e.getMember().getColor());
                            currentChannel.sendMessage(eb.build()).queue();
                        }
                    }
                } catch (Exception ignored) {}
            }
        } catch (Exception ignored) {}
    }

}
