package xyz.sheepstar.wrapper.core;

import drivers.MySQLConnection;
import net.dv8tion.jda.api.JDA;
import org.apache.commons.cli.*;
import org.discordbots.api.client.DiscordBotListAPI;
import xyz.sheepstar.wrapper.main.commands.essentials.BotInfoCommand;
import xyz.sheepstar.wrapper.main.commands.essentials.HelpCommand;
import xyz.sheepstar.wrapper.main.commands.essentials.PingCommand;
import xyz.sheepstar.wrapper.main.commands.essentials.UptimeCommand;
import xyz.sheepstar.wrapper.main.commands.modules.*;
import xyz.sheepstar.wrapper.main.commands.trending.LeaderboardCacheUpdater;
import xyz.sheepstar.wrapper.main.commands.trending.LeaderboardCommand;
import xyz.sheepstar.wrapper.util.Config;
import xyz.sheepstar.wrapper.util.api.GuildManager;
import xyz.sheepstar.wrapper.util.api.autoupdater.AutoUpdate;
import xyz.sheepstar.wrapper.util.api.http.WebServer;
import xyz.sheepstar.wrapper.util.api.module.ModuleManager;
import xyz.sheepstar.wrapper.util.api.settings.SettingsManager;

/********************************
 * @author Mathias Wagner
 * Created 22.01.2021
 ********************************/

public class Main {

    private static MySQLConnection mySQLConnection;
    private static Config sheepstarConfig;
    private static DiscordBotListAPI botListAPI;
    private static CommandLine cmd;
    private static JDA jda;

    // Managers
    private static GuildManager guildManager = new GuildManager();
    private static ModuleManager moduleManager = new ModuleManager();
    private static SettingsManager settingsManager = new SettingsManager();
    public static void main(String[] args) throws Exception {

        // Initialize Sheepstar
        Bootstrap bootstrap = new Bootstrap(args).init();
        mySQLConnection = bootstrap.getMySQLConnection();
        sheepstarConfig = bootstrap.getConfig();
        cmd = bootstrap.getArgs();
        jda = bootstrap.getJDA();
        jda.awaitReady();

        // Start Webserver
        new WebServer().start();

        // Start AutoUpdate
        new AutoUpdate().start();

        // New Leaderboard Updater
        new LeaderboardCacheUpdater().start();

        // Register Sheepstar Commands
        jda.addEventListener(new PingCommand());
        jda.addEventListener(new HelpCommand());
        jda.addEventListener(new EnableModuleCommand());
        jda.addEventListener(new ListModulesCommand());
        jda.addEventListener(new ModuleInfoCommand());
        jda.addEventListener(new DisableModuleCommand());
        jda.addEventListener(new BotInfoCommand());
        jda.addEventListener(new ShowModulesCommand());
        jda.addEventListener(new UptimeCommand());
        jda.addEventListener(new CommandInfoCommand());
        jda.addEventListener(new LeaderboardCommand());

        // Register Sheepstar Modules
        bootstrap.enableAllModules("xyz.sheepstar.wrapper.modules");
    }

    public static CommandLine getArgs() { return cmd; }

    public static MySQLConnection getDB() {
        return mySQLConnection;
    }

    public static GuildManager getGuildManager() {
        return guildManager;
    }

    public static ModuleManager getModuleManager() {
        return moduleManager;
    }

    public static SettingsManager getSettingsManager() {
        return settingsManager;
    }

    public static Config getConfig() {
        return sheepstarConfig;
    }

    public static JDA getJDA() {
        return jda;
    }
}
