package xyz.sheepstar.wrapper.main.commands.essentials;

import net.dv8tion.jda.api.events.message.guild.GuildMessageReceivedEvent;
import xyz.sheepstar.wrapper.util.api.command.GuildCommand;
import xyz.sheepstar.wrapper.util.api.permission.PermissionNode;

import java.lang.management.ManagementFactory;
import java.lang.management.RuntimeMXBean;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Collections;
import java.util.Date;
import java.util.List;

/********************************
 * @author Mathias Wagner
 * Created 10.02.2021
 ********************************/

public class UptimeCommand extends GuildCommand {

    private static final long SECOND = 1000;
    private static final long MINUTE = 60 * SECOND;
    private static final long HOUR = 60 * MINUTE;
    private static final long DAY = 24 * HOUR;

    @Override
    public List<String> getAlias() {
        return Collections.singletonList("uptime");
    }

    @Override
    public PermissionNode getPermission() {
        return null;
    }

    @Override
    public void executeCommand(String[] args, GuildMessageReceivedEvent e) {
        RuntimeMXBean rb = ManagementFactory.getRuntimeMXBean();

        sendMessage()
                .setTitle("Sheepstar uptime")
                .setDescription(formatUptime(rb.getUptime()))
                .send(e.getChannel());
    }


    public static String formatUptime(long uptime) {
        StringBuilder buf = new StringBuilder();
        if (uptime > DAY) {
            long days = (uptime - uptime % DAY) / DAY;
            buf.append(days);
            buf.append(" Days");
            uptime = uptime % DAY;
        }
        if (uptime > HOUR) {
            long hours = (uptime - uptime % HOUR) / HOUR;
            if (buf.length() > 0) buf.append(", ");
            buf.append(hours);
            buf.append(" Hours");
            uptime = uptime % HOUR;
        }
        if (uptime > MINUTE) {
            long minutes = (uptime - uptime % MINUTE) / MINUTE;
            if (buf.length() > 0) buf.append(", ");

            buf.append(minutes);
            buf.append(" Minutes");
            uptime = uptime % MINUTE;
        }
        if (uptime > SECOND) {
            long seconds = (uptime - uptime % SECOND) / SECOND;
            if (buf.length() > 0) buf.append(", ");
            buf.append(seconds);
            buf.append(" Seconds");
            uptime = uptime % SECOND;
        }
        if (uptime > 0) {
            if (buf.length() > 0)  buf.append(", ");
            buf.append(uptime);
            buf.append(" Milliseconds");
        }
        return buf.toString();
    }
}
