package xyz.sheepstar.wrapper.modules.counter.action;

import net.dv8tion.jda.api.entities.*;
import xyz.sheepstar.wrapper.core.Main;

import java.util.ArrayList;
import java.util.List;
import java.util.Objects;
import java.util.concurrent.Executors;
import java.util.concurrent.ScheduledExecutorService;
import java.util.concurrent.TimeUnit;

/********************************
 * @author Mathias Wagner
 * Created 08.02.2021
 ********************************/

public class MemberUpdate {

    public void run() {
        Runnable drawRunnable = new Runnable() {
            public void run() {
                ArrayList<String> currentGuilds = new ArrayList<>();
                Main.getDB().selectFrom("counter_channels").getResult().getList("guildID").forEach(guildID -> {
                    if (!currentGuilds.contains(guildID)) {
                        currentGuilds.add(guildID);
                        try { update(Objects.requireNonNull(Main.getJDA().getGuildById(guildID))); } catch (Exception ignored) { }
                    }
                });
            }};
        ScheduledExecutorService exec = Executors.newScheduledThreadPool(1);
        exec.scheduleAtFixedRate(drawRunnable , 0, 10, TimeUnit.MINUTES);
    }

    public void update(Guild guild) {
        Main.getDB().selectFrom("counter_channels").where("guildID", guild.getId()).getResult().getMultipleList("keyName", "value").forEach((monitorType, channelID) -> {
            // Update Member Monitors
            if (monitorType.equals("member_id") || monitorType.equals("allmembers_id") || monitorType.equals("bots_id")) {
                guild.loadMembers().onSuccess(list -> updateMemberMonitor(monitorType, list, channelID));
            }
            // Update Channel Monitors
            if (monitorType.equals("channels_id") || monitorType.equals("textchannels_id") || monitorType.equals("voicechannels_id")) {
                updateChannelMonitor(monitorType, guild.getChannels(), channelID);
            }
        });
    }

    public void updateChannelMonitor(String monitorType, List<GuildChannel> guilds, String channelID) {
        if (monitorType.equals("voicechannels_id"))
            updateChannelName(channelID, guilds.stream().filter(guild -> guild instanceof VoiceChannel).count());
        if (monitorType.equals("textchannels_id"))
            updateChannelName(channelID,guilds.stream().filter(guild -> guild instanceof TextChannel).count());
        if (monitorType.equals("channels_id"))
            updateChannelName(channelID, guilds.size());
    }

    public void updateMemberMonitor(String monitorType, List<Member> members, String channelID) {
        if (monitorType.equals("member_id"))
            updateChannelName(channelID, members.stream().filter(member -> !member.getUser().isBot()).count());
        if (monitorType.equals("bots_id"))
            updateChannelName(channelID, members.stream().filter(member -> member.getUser().isBot()).count());
        if (monitorType.equals("allmembers_id")) {
            updateChannelName(channelID, members.size());
        }
    }

    public void updateChannelName(String channelID, long count) {
        try{
            GuildChannel channel = Main.getJDA().getGuildChannelById(channelID);
            channel.getManager().setName(channel.getName().replaceAll("\\d+", ""+count)).queue();
        } catch (NullPointerException err) {
            Main.getDB().updateTable("counter_channels").where("value", channelID).delete();
        }
    }

}
