package xyz.sheepstar.wrapper.modules.counter.commands;

import net.dv8tion.jda.api.events.message.guild.GuildMessageReceivedEvent;
import xyz.sheepstar.wrapper.modules.counter.action.MemberUpdate;
import xyz.sheepstar.wrapper.modules.counter.api.Manager;
import xyz.sheepstar.wrapper.util.api.command.GuildCommand;
import xyz.sheepstar.wrapper.util.api.permission.PermissionNode;
import xyz.sheepstar.wrapper.util.api.sql.StorageMedium;

import java.util.Arrays;
import java.util.List;

/********************************
 * @author Mathias Wagner
 * Created 08.02.2021
 ********************************/

public class EnableCommand extends GuildCommand {
    @Override
    public List<String> getAlias() {
        return Arrays.asList("enablemodule", "enable", "activate", "activatemodule");
    }

    @Override
    public PermissionNode getPermission() {
        return PermissionNode.ADMINISTRATOR;
    }

    @Override
    public void executeCommand(String[] args, GuildMessageReceivedEvent e) {
        StorageMedium storage = new StorageMedium(e.getGuild(), "counter_channels");
        if (args.length != 2) {
            sendUsage(e, "enable <monitorName> <text/voice>\nYou can find a list under " + moduleManager.getPrefix(e.getGuild(), "counter")+"list");
            return;
        }

        String monitorName = args[0].toLowerCase();
        if (Manager.monitorList.contains(monitorName.toLowerCase())) {
            sendErrorMessage().setDescription("This module does not exist. Use a module from " + moduleManager.getPrefix(e.getGuild(), "counter")+"list");
            return;
        }
        String channelType = args[1].toLowerCase();
        if (!(channelType.equalsIgnoreCase("text") || channelType.equalsIgnoreCase("voice"))) {
            sendErrorMessage().setDescription("Please use a valid channel type (voice/text)").send(e.getChannel());
            return;
        }

        if (!storage.exists(monitorName+"_id")) {
            if (channelType.equals("voice")) {
                e.getGuild().createVoiceChannel("\uD83D\uDC65▎%channel%: 0".replace("%channel%", toCamelCase(monitorName))).setPosition(0).queue(voiceChannel -> {
                    storage.insert(monitorName+"_id", voiceChannel.getId());
                    new MemberUpdate().update(e.getGuild());
                });
            } else if (channelType.equals("text")) {
                e.getGuild().createTextChannel("\uD83D\uDC65▎%channel%:0".replace("%channel%", monitorName)).setPosition(0).queue(textChannel -> {
                    storage.insert(monitorName+"_id", textChannel.getId());
                    new MemberUpdate().update(e.getGuild());
                });
            }
            sendMessage()
                    .setDescription("You have successfully created a "+monitorName+" channel!").send(e.getChannel());
        } else {
            sendErrorMessage().setDescription("You already added this channel").send(e.getChannel());
        }

    }
}
