package xyz.sheepstar.wrapper.modules.global.api;

import net.dv8tion.jda.api.entities.Guild;
import net.dv8tion.jda.api.entities.TextChannel;
import xyz.sheepstar.wrapper.core.Main;

/********************************
 * @author Mathias Wagner
 * Created 09.02.2021
 ********************************/

public class GlobalManager {

    public boolean isRegistered(Guild guild, TextChannel channel) {
        return Main.getDB().getResult("SELECT * FROM global_channels WHERE guildID=? AND channelID=?", guild.getId(), channel.getId()).getRowCount() == 1;
    }

    public void register(Guild guild, TextChannel channel) {
        if (isRegistered(guild, channel)) return;
        Main.getDB().update("INSERT INTO global_channels (guildID, channelID, language, roomID) VALUES (?, ?, 'EN', '1')", guild.getId(), channel.getId());
    }

    public void unregister(Guild guild, TextChannel channel) {
        if (!isRegistered(guild, channel)) return;
        Main.getDB().update("DELETE FROM global_channels WHERE guildID=? AND channelID=?", guild.getId(), channel.getId());
    }

    public boolean isValidRoom(String roomCode) {
        return Main.getDB().getResult("SELECT * FROM global_rooms WHERE code=?", roomCode).getRowCount() == 1;
    }

    public boolean isValidLanguage(String languageCode) {
        return Main.getDB().getResult("SELECT * FROM global_languages WHERE code=?", languageCode).getRowCount() == 1;
    }

    public void changeLanguage(TextChannel channel, String newLanguage) {
        if (!isValidLanguage(newLanguage)) return;
        Main.getDB().update("UPDATE global_channels SET language=? WHERE channelID=?", newLanguage.toUpperCase(), channel.getId());
    }

    public void changeRoom(TextChannel channel, String newRoom) {
        if (!isValidRoom(newRoom)) return;
        Main.getDB().update("UPDATE global_channels SET roomID=? WHERE channelID=?", newRoom.toLowerCase(), channel.getId());
    }

    public String getRoom(TextChannel channel) {
        return Main.getDB().getResult("SELECT * FROM global_channels WHERE channelID=?", channel.getId()).getString("roomID");
    }

    public String getLanguage(TextChannel channel) {
        return Main.getDB().getResult("SELECT * FROM global_channels WHERE channelID=?", channel.getId()).getString("language");
    }


}
