package xyz.sheepstar.wrapper.modules.global.listeners;

import net.dv8tion.jda.api.entities.Guild;
import net.dv8tion.jda.api.entities.TextChannel;
import net.dv8tion.jda.api.events.message.guild.GuildMessageReceivedEvent;
import org.jetbrains.annotations.NotNull;
import xyz.sheepstar.wrapper.core.Main;
import xyz.sheepstar.wrapper.modules.global.api.GlobalCommand;
import xyz.sheepstar.wrapper.modules.global.api.GlobalManager;
import xyz.sheepstar.wrapper.modules.global.api.ValidationCheck;
import xyz.sheepstar.wrapper.util.api.builder.message.MessageBuilder;
import xyz.sheepstar.wrapper.util.api.listener.GuildListener;

import java.sql.ResultSet;
import java.time.OffsetDateTime;

/********************************
 * @author Mathias Wagner
 * Created 09.02.2021
 ********************************/

public class GlobalListener extends GuildListener {

    @Override
    public void onGuildMessageReceived(@NotNull GuildMessageReceivedEvent e) {
        GlobalManager gm = GlobalCommand.globalManager;
        if (!isValid(e.getGuild())) return;
        if (!gm.isRegistered(e.getGuild(), e.getChannel())) return;
        if (!new ValidationCheck().validate(e)) return;

        String roomID = gm.getRoom(e.getChannel());
        String language = gm.getLanguage(e.getChannel());

        // NSFW check
        if (!e.getChannel().isNSFW() && roomID.contains("n")) {
            sendErrorMessage()
                    .setDescription("You joined an NSFW Room. Please set this Channel to NSFW or change the Room with `s!rooms`")
                    .send(e.getChannel());
        }

        // TODO:
        sendGlobal(e, language, roomID);
    }


    public void sendGlobal(GuildMessageReceivedEvent e, String language, String roomID) {
        GlobalManager gm = GlobalCommand.globalManager;
        ResultSet rs = Main.getDB().getResult("SELECT * FROM global_channels WHERE language=? AND roomID=?", language, roomID).getResultSet();
        try {
            while (rs.next()) {
                try {
                    String guildID = rs.getString("guildID");
                    String channelID = rs.getString("channelID");
                    Guild currentGuild = Main.getJDA().getGuildById(guildID);
                    TextChannel currentChannel = currentGuild.getTextChannelById(channelID);
                    if (!isValid(e.getGuild())) break;
                    MessageBuilder mb = sendMessage();
                    String iconURL = e.getGuild().getIconUrl();
                    if (iconURL == null) iconURL = "https://i.giphy.com/media/38pPhtupwYMerEQeBT/source.gif";
                    mb.setThumbnail(iconURL);
                    mb.enableTimeStamp();
                    if (roomID.startsWith("n")) mb.addField(" ", "`NSFW` :underage:", false);

                    mb.setFooter("Sent by " + e.getGuild().getName(), iconURL);

                    //https://github.com/sheepstar-dc/global-old/blob/master/src/main/java/de/sheepstar/globalWrapper/listeners/GlobalListener.java

                    String links = "[Support Server](https://discord.gg/jbne9JTJtU) ║ [Bot Invite](https://discord.com/api/oauth2/authorize?client_id=772513556116930580&permissions=321536&scope=bot)";
                    mb.addField("", links, false);

                    if (e.getMessage().getAttachments().size() > 0)
                        mb.setImage(e.getMessage().getAttachments().get(0).getUrl());

                    mb.setDescription(e.getMessage().getContentRaw());
                    mb.setAuthor(e.getAuthor().getAsTag(), e.getAuthor().getAvatarUrl());

                    mb.send(currentChannel);
                } catch (Exception ignored) {}
            }
        } catch (Exception ignored) {}

    }

}
