package xyz.sheepstar.wrapper.modules.voice.listeners;

import net.dv8tion.jda.api.entities.Category;
import net.dv8tion.jda.api.entities.Guild;
import net.dv8tion.jda.api.entities.Member;
import net.dv8tion.jda.api.entities.VoiceChannel;
import net.dv8tion.jda.api.events.Event;
import net.dv8tion.jda.api.events.guild.GenericGuildEvent;
import net.dv8tion.jda.api.events.guild.voice.GuildVoiceJoinEvent;
import net.dv8tion.jda.api.events.guild.voice.GuildVoiceLeaveEvent;
import net.dv8tion.jda.api.events.guild.voice.GuildVoiceMoveEvent;
import net.dv8tion.jda.api.events.message.guild.GuildMessageReceivedEvent;
import net.dv8tion.jda.api.hooks.ListenerAdapter;
import org.jetbrains.annotations.NotNull;
import xyz.sheepstar.wrapper.modules.voice.api.Manager;
import xyz.sheepstar.wrapper.modules.voice.api.TempVoiceManager;
import xyz.sheepstar.wrapper.modules.voice.api.VoiceManager;
import xyz.sheepstar.wrapper.util.api.listener.GuildListener;

/********************************
 * @author Mathias Wagner
 * Created 25.01.2021
 ********************************/

public class JoinListener extends GuildListener {

    private TempVoiceManager tempVoiceManager = Manager.getTempVoiceManager();
    private VoiceManager voiceManager = Manager.getVoiceManager();

    @Override
    public void onGuildVoiceJoin(@NotNull GuildVoiceJoinEvent e) {
        if (isValid(e.getGuild())) join(e.getChannelJoined(), e.getMember());
    }

    @Override
    public void onGuildVoiceLeave(@NotNull GuildVoiceLeaveEvent e) {
        if (isValid(e.getGuild())) leave(e.getChannelLeft());
    }

    @Override
    public void onGuildVoiceMove(@NotNull GuildVoiceMoveEvent e) {
        if (!isValid(e.getGuild())) return;
        leave(e.getChannelLeft());
        join(e.getChannelJoined(), e.getMember());
    }

    public void join(VoiceChannel joined, Member memb) {
        if (voiceManager.isRegistered(joined)) {
            Category cat = joined.getParent();
            String origin = voiceManager.getChannelName(joined)
                    .replace("%name%", memb.getEffectiveName())
                    .replace("%tag%", memb.getUser().getAsTag().split("#")[1]);
            cat.createVoiceChannel(origin).queue(vc -> {
                vc.getManager().setUserLimit(joined.getUserLimit()).queue();
                Guild guild = vc.getGuild();
                guild.moveVoiceMember(memb, vc).queue();
                tempVoiceManager.register(guild, vc);
            });
        }
    }

    public void leave(VoiceChannel channel) {
        if (channel.getMembers().size() <= 0) {
            if(tempVoiceManager.isRegistered(channel)) {
                channel.delete().queue();
                tempVoiceManager.unregister(channel);
            }
        }
    }

}
