package xyz.sheepstar.wrapper.util.api;

import com.google.common.base.CaseFormat;
import net.dv8tion.jda.api.JDA;
import net.dv8tion.jda.api.entities.Emote;
import net.dv8tion.jda.api.entities.Guild;
import net.dv8tion.jda.api.events.message.MessageReceivedEvent;
import net.dv8tion.jda.api.events.message.guild.GuildMessageReceivedEvent;
import net.dv8tion.jda.api.hooks.ListenerAdapter;
import org.apache.commons.lang3.StringUtils;
import xyz.sheepstar.wrapper.core.Main;
import xyz.sheepstar.wrapper.util.api.builder.message.MessageBuilder;
import xyz.sheepstar.wrapper.util.api.builder.message.MessageType;
import xyz.sheepstar.wrapper.util.api.command.GuildCommand;
import xyz.sheepstar.wrapper.util.api.module.ModuleManager;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Random;

/********************************
 * @author Mathias Wagner
 * Created 09.02.2021
 ********************************/

public class GuildEssentials extends ListenerAdapter {

    public JDA jda = Main.getJDA();

    public ModuleManager moduleManager = Main.getModuleManager();
    public GuildManager guildManager = Main.getGuildManager();


    public MessageBuilder sendMessage() {
        return new MessageBuilder(MessageType.SUCCESS).setThumbnail(getModuleIcon());
    }

    public MessageBuilder sendErrorMessage() {
        return new MessageBuilder(MessageType.ERROR).setThumbnail(getModuleIcon());
    }

    // TODO: translate
    public void sendPlainMessage(MessageReceivedEvent e, String msg) {
        e.getChannel().sendMessage(msg).queue();
    }

    public void sendPlainMessage(GuildMessageReceivedEvent e, String msg) {
        e.getChannel().sendMessage(msg).queue();
    }

    public String toCamelCase(String msg) {
        return CaseFormat.UPPER_UNDERSCORE.to(CaseFormat.UPPER_CAMEL, msg);
    }

    public String getEmoteString(String emoteName) {
        try {
            return getEmote(emoteName).getAsMention();
        } catch (Exception err) { return ":star:"; }
    }

    public Emote getEmote(String emoteName) {
        try {
            return jda.getEmotesByName(emoteName, true).get(0);
        } catch (Exception err) {
            return jda.getEmotesByName(":star:", true).get(0);
        }
    }

    public boolean isValid(Guild guild) {
        String packageName = this.getClass().getPackage().getName();
        if (packageName.startsWith("xyz.sheepstar.wrapper.modules") && packageName.endsWith(".listeners") || packageName.endsWith(".commands")) {
            packageName = packageName.replace("xyz.sheepstar.wrapper.modules.", "");
            if (packageName.split("\\.").length >= 1) packageName = packageName.split("\\.")[0];
            if (!moduleManager.exists(packageName)) return false;
            if (!moduleManager.isEnabled(guild, packageName)) return false;
            return true;
        } else return false;
    }

    public String getModuleName() {
        String packageName = this.getClass().getPackage().getName();
        if (packageName.startsWith("xyz.sheepstar.wrapper.modules") && packageName.endsWith(".listeners") || packageName.endsWith(".commands")) {
            packageName = packageName.replace("xyz.sheepstar.wrapper.modules.", "");
            if (packageName.split("\\.").length >= 1) packageName = packageName.split("\\.")[0];
            return packageName;
        } else return "default";
    }

    public String getModuleIcon() {
        String moduleName = getModuleName();
        if (!moduleName.equals("default")) {
            String icon = Main.getDB().getResult("SELECT * FROM existing_modules WHERE name=?", moduleName).getString("icon_url");
            if (!icon.startsWith("http")) return "https://sheepstar.xyz/assets/favicon.png";
            return icon;
        } else return "https://sheepstar.xyz/assets/favicon.png";
    }

    public HashMap<String, String> getCommands(String moduleName) {
        HashMap<String, String> aliases = new HashMap<>();

        jda.getRegisteredListeners().forEach(obj -> {
            if (obj.getClass().getName().startsWith("xyz.sheepstar.wrapper.modules."+moduleName+".commands")) {
                GuildCommand cmd = (GuildCommand) obj;
                String className = obj.getClass().getSimpleName().replace("Command", "");
                className = StringUtils.join(StringUtils.splitByCharacterTypeCamelCase(className.replaceAll("\\d+", "")), " ");
                aliases.put(className, cmd.getAlias().get(0));
            }
        });
        return aliases;
    }

}
