package xyz.sheepstar.wrapper.util.api.builder.message;

import net.dv8tion.jda.api.EmbedBuilder;
import net.dv8tion.jda.api.entities.MessageEmbed;
import net.dv8tion.jda.api.entities.TextChannel;
import xyz.sheepstar.wrapper.util.api.GuildEssentials;

import java.awt.*;
import java.time.OffsetDateTime;
import java.time.ZoneId;

/********************************
 * @author Mathias Wagner
 * Created 24.01.2021
 ********************************/

public class MessageBuilder extends GuildEssentials {

    private EmbedBuilder eb;

    public MessageBuilder(MessageType type) {
        eb = new EmbedBuilder();
        if (type == MessageType.SUCCESS) {
            setColor(ColorType.GREEN);
        } else if (type == MessageType.WARNING) {
            setColor(ColorType.ORANGE);
        } else if (type == MessageType.ERROR) {
            setColor(ColorType.RED);
        }
    }

    public MessageBuilder addField(String name, String value, boolean inline) {
        eb.addField(name, value, inline);
        return this;
    }

    public MessageBuilder setTitle(String title) {
        eb.setTitle(title);
        return this;
    }

    public MessageBuilder setColor(ColorType colorType) {
        eb.setColor(colorType.getColor());
        return this;
    }

    public MessageBuilder setColor(Color color) {
        eb.setColor(color);
        return this;
    }

    public MessageBuilder setDescription(String description) {
        eb.setDescription(description);
        return this;
    }

    public MessageBuilder setAuthor(String author) {
        eb.setAuthor(author);
        return this;
    }

    public MessageBuilder setAuthor(String author, String url) {
        eb.setAuthor(author, null, url);
        return this;
    }

    public MessageBuilder setThumbnail(String url) {
        eb.setThumbnail(url);
        return this;
    }

    public MessageBuilder noThumbnail() {
        eb.setThumbnail(null);
        return this;
    }

    public MessageBuilder setFooter(String footer) {
        eb.setFooter(footer);
        return this;
    }

    public MessageBuilder setFooter(String footer, String url) {
        eb.setFooter(footer, url);
        return this;
    }

    public MessageBuilder enableTimeStamp() {
        eb.setTimestamp(OffsetDateTime.now().atZoneSameInstant(ZoneId.of("Europe/Berlin")));
        return this;
    }

    public MessageEmbed get() {
        return eb.build();
    }

    public MessageBuilder send(TextChannel channel) {
        channel.sendMessage(eb.build()).queue();
        return this;
    }

    public MessageBuilder setImage(String url) {
        eb.setImage(url);
        return this;
    }
}
