package xyz.sheepstar.wrapper.util.api.command;

import net.dv8tion.jda.api.JDA;
import net.dv8tion.jda.api.entities.Guild;
import net.dv8tion.jda.api.entities.Message;
import net.dv8tion.jda.api.events.message.guild.GuildMessageReceivedEvent;
import xyz.sheepstar.wrapper.core.Main;
import xyz.sheepstar.wrapper.util.api.GuildManager;
import xyz.sheepstar.wrapper.util.api.module.ModuleManager;
import xyz.sheepstar.wrapper.util.api.permission.PermissionNode;

import java.util.ArrayList;
import java.util.List;

/********************************
 * @author Mathias Wagner
 * Created 23.01.2021
 ********************************/

public abstract class GuildCommand extends Command {

    public abstract List<String> getAlias();
    public abstract PermissionNode getPermission();
    public abstract void executeCommand(String[] args, GuildMessageReceivedEvent e);

    public ModuleManager moduleManager = Main.getModuleManager();
    public GuildManager guildManager = Main.getGuildManager();

    public String getPrefix(Guild guild) { return Main.getGuildManager().getPrefix(guild); }

    public String getModulePrefix(Guild guild, String moduleName) { return moduleManager.getPrefix(guild, moduleName); }

    @Override
    public void onGuildMessageReceived( GuildMessageReceivedEvent e) {
        if (e.getAuthor() == null || e.getChannel() == null) return;
        if (e.getAuthor().isBot()) return;
        String packageName = this.getClass().getPackage().getName();
        if (packageName.startsWith("xyz.sheepstar.wrapper.modules.")) {
            packageName = packageName.replace("xyz.sheepstar.wrapper.modules.", "");
            if (packageName.split("\\.").length >= 1) packageName = packageName.split("\\.")[0];
            if (!moduleManager.exists(packageName)) return;
            if (!isValidCommand(e.getGuild(), e.getMessage(), getModulePrefix(e.getGuild(), packageName))) return;
            executeCommand( commandArgs( e.getGuild(), e.getMessage() ), e );
        } else {
            if (!isValidCommand(e.getGuild(), e.getMessage(), getPrefix(e.getGuild()))) return;
            executeCommand( commandArgs( e.getGuild(), e.getMessage() ), e );
        }
    }

    public void sendUsage(GuildMessageReceivedEvent e, String usage) {
        sendErrorMessage()
                .setTitle("Wrong usage")
                .setDescription("Use " + getPrefix(e.getGuild()) + getAlias().get(0) + " " + usage)
                .send(e.getChannel());
    }

    private boolean isValidCommand(Guild guild, Message msg, String prefix) {
        if (!msg.getContentRaw().startsWith(prefix)) return false;
        String cmdName = msg.getContentRaw().substring(prefix.length()).toLowerCase();
        if (cmdName.contains(" ")) cmdName = cmdName.substring(0, cmdName.indexOf(" "));
        if (cmdName.contains("\n")) cmdName = cmdName.substring(0, cmdName.indexOf("\n"));
        return getAlias().contains(cmdName.toLowerCase());
    }

    private String[] commandArgs(Guild guild, Message msg) {
        String prefix = getPrefix(guild);
        String noPrefix = msg.getContentRaw().substring(prefix.length());
        if (!noPrefix.contains(" ")) return new String[]{};
        return noPrefix.substring(noPrefix.indexOf(" ") + 1).split("\\s+");
    }

}
