package xyz.sheepstar.wrapper.util.api.sql;

import drivers.MySQLConnection;
import fields.SQLField;
import generator.TableGenerator;
import net.dv8tion.jda.api.entities.Guild;
import types.SQLType;
import xyz.sheepstar.wrapper.core.Main;

/********************************
 * @author Mathias Wagner
 * Created 07.02.2021
 ********************************/

public class StorageMedium {

    private Guild guild;
    private String tableName;

    public StorageMedium(Guild guild, String tableName) {
        this.guild = guild;
        this.tableName = tableName;
        this.createTable();
    }

    public StorageMedium(String tableName) {
        this.tableName = tableName;
        this.createTable();
    }

    private void createTable() {
        TableGenerator tg = Main.getDB().updateTable(tableName).generateTable().useID();
        if (this.guild != null) tg.addField(new SQLField(SQLType.STRING, "guildID", 255));
        tg.addField(new SQLField(SQLType.STRING, "keyName", 255))
                .addField(new SQLField(SQLType.STRING, "value", 255))
                .create();
    }

    public boolean exists(String key) {
        return this.guild != null ?
                Main.getDB().selectFrom(this.tableName).where("keyName", key).where("guildID", guild.getId()).getResult().getRowCount() != 0
                : Main.getDB().selectFrom(this.tableName).where("keyName", key).getResult().getRowCount() != 0;
    }

    public boolean existsValue(String value) {
        return this.guild != null ?
                Main.getDB().selectFrom(this.tableName).where("value", value).where("guildID", guild.getId()).getResult().getRowCount() != 0
                : Main.getDB().selectFrom(this.tableName).where("value", value).getResult().getRowCount() != 0;
    }

    public void insert(String key, Object value) {
        if (this.guild != null) {
            Main.getDB().update("INSERT INTO " + this.tableName + " (guildID, keyName, value) VALUES (?, ?, ?)", guild.getId(), key, value + "");
        } else {
            Main.getDB().update("INSERT INTO " + this.tableName + " (keyName, value) VALUES (?, ?)", key, value + "");
        }
    }

    public void delete(String key) {
        if (this.guild != null) {
            Main.getDB().updateTable(this.tableName).where("keyName", key).where("guildID", this.guild.getId()).delete();
        } else {
            Main.getDB().updateTable(this.tableName).where("keyName", key).delete();
        }
    }

    public void deleteValue(String value) {
        if (this.guild != null) {
            Main.getDB().updateTable(this.tableName).where("value", value).where("guildID", this.guild.getId()).delete();
        } else {
            Main.getDB().updateTable(this.tableName).where("value", value).delete();
        }
    }

    public void update(String key, String newValue) {
        if (this.guild != null) {
            Main.getDB().update("UPDATE " + this.tableName + " SET value=? WHERE guildID=? AND keyName=?", newValue, guild.getId(), key);
        } else {
            Main.getDB().update("UPDATE " + this.tableName + " SET value=? WHERE keyName=?", key);
        }
    }

    public void inUpdate(String key, String newValue) {
        if (exists(key)) update(key, newValue); else insert(key, newValue);
    }

    public Object getValue(String key) {
        if (!exists(key)) return "0";
        return (guild != null) ? Main.getDB().selectFrom(this.tableName).where("keyName", key).where("guildID", guild.getId()).getResult().getObject("value")
                : Main.getDB().selectFrom(this.tableName).where("keyName", key).getResult().getObject("value");
    }

    public int getCount(String key) {
        return this.guild != null ?
                Main.getDB().selectFrom(this.tableName).where("keyName", key).where("guildID", guild.getId()).getResult().getRowCount()
                : Main.getDB().selectFrom(this.tableName).where("keyName", key).getResult().getRowCount();
    }



}
